package de.ozgcloud.nachrichten.postfach.osiv2.transfer;

import java.time.ZoneOffset;

import org.jsoup.Jsoup;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.ReportingPolicy;

import de.ozgcloud.nachrichten.postfach.PostfachAddress;
import de.ozgcloud.nachrichten.postfach.PostfachAddressIdentifier;
import de.ozgcloud.nachrichten.postfach.PostfachNachricht;
import de.ozgcloud.nachrichten.postfach.osiv2.OsiPostfachRemoteService;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.V1ReplyBehavior;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.V1ReplyMessage;
import lombok.Builder;
import lombok.Getter;

@Mapper(unmappedTargetPolicy = ReportingPolicy.ERROR, imports = { ZoneOffset.class, Osi2HtmlDocument.class })
public interface Osi2ResponseMapper {

	String POSTFACH_ADDRESS_VERSION = "2.0";
	int POSTFACH_ADDRESS_TYPE = 2;

	@Mapping(target = "id", source = "guid")
	@Mapping(target = "postfachAddress", expression = "java(buildPostfachAddressByPostfachId(message.getMessageBox().toString()))")
	@Mapping(target = "messageId", ignore = true)
	@Mapping(target = "createdAt", expression = "java(java.time.ZonedDateTime.now())")
	@Mapping(target = "createdBy", source = "displayName")
	@Mapping(target = "sentAt", expression = "java(message.getResponseTime().toZonedDateTime())")
	@Mapping(target = "sentSuccessful", ignore = true)
	@Mapping(target = "messageCode", ignore = true)
	@Mapping(target = "direction", constant = "IN")
	@Mapping(target = "vorgangId", source = "sequencenumber")
	@Mapping(target = "referencedNachricht", ignore = true)
	@Mapping(target = "mailBody", expression = "java( message.getIsHtml() ? Osi2HtmlDocument.renderToPlainText(message.getBody()) : message.getBody() )")
	@Mapping(target = "replyOption", source = "replyAction")
	@Mapping(target = "attachments", ignore = true)
	PostfachNachricht toPostfachNachricht(V1ReplyMessage message);

	default PostfachAddress buildPostfachAddressByPostfachId(String postfachId) {
		return PostfachAddress.builder()
				.type(POSTFACH_ADDRESS_TYPE)
				.version(POSTFACH_ADDRESS_VERSION)
				.identifier(StringBasedIdentifier.builder().mailboxId(postfachId).build())
				.serviceKontoType(OsiPostfachRemoteService.POSTFACH_TYPE_OSI)
				.build();

	}

	default PostfachNachricht.ReplyOption mapReplyAction(V1ReplyBehavior replyOption) {
		return switch (replyOption) {
			case REPLYFORBIDDEN -> PostfachNachricht.ReplyOption.FORBIDDEN;
			case REPLYPOSSIBLE -> PostfachNachricht.ReplyOption.POSSIBLE;
			case REPLYMANDATORY -> PostfachNachricht.ReplyOption.MANDATORY;
		};
	}

	@Builder
	@Getter
	class StringBasedIdentifier implements PostfachAddressIdentifier {

		private String mailboxId;

		@Override
		public boolean isStringBasedIdentifier() {
			return true;
		}

		@Override
		public String toString() {
			return mailboxId;
		}
	}
}
