package de.ozgcloud.nachrichten.postfach.osiv2.transfer;

import static org.assertj.core.api.AssertionsForInterfaceTypes.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;

class Osi2HtmlDocumentTest {

	@DisplayName("should render anchor link")
	@Test
	void shouldRenderAnchorLink() {
		var plainText = renderPlainText("<a href=\"https://www.example.com\">Example</a>");

		assertThat("[Example](https://www.example.com)").isEqualTo(plainText);
	}

	@DisplayName("should render h1 heading")
	@Test
	void shouldRenderH1Heading() {
		var plainText = renderPlainText("<h1>Example</h1>");

		assertThat(plainText).isEqualTo("# Example");
	}

	@DisplayName("should render h2 heading")
	@Test
	void shouldRenderH2Heading() {
		var plainText = renderPlainText("<h2>Example</h2>");

		assertThat(plainText).isEqualTo("## Example");
	}

	@DisplayName("should render h3  heading")
	@Test
	void shouldRenderH3Heading() {
		var plainText = renderPlainText("<h3>Example</h3>");

		assertThat(plainText ).isEqualTo("### Example");
	}


	@DisplayName("should render italic")
	@ParameterizedTest
	@ValueSource(strings = { "<i>Example</i>", "<em>Example</em>" })
	void shouldRenderItalic(String italicExample) {
		var plainText = renderPlainText(italicExample);

		assertThat(plainText).isEqualTo("_Example_");
	}

	@DisplayName("should render bold")
	@ParameterizedTest
	@ValueSource(strings = { "<b>Example</b>", "<strong>Example</strong>" })
	void shouldRenderBold(String boldExample) {
		var plainText = renderPlainText(boldExample);

		assertThat(plainText).isEqualTo("__Example__");
	}

	@DisplayName("should render image")
	@Test
	void shouldRenderImage() {
		var plainText = renderPlainText("<img src=\"https://www.example.com/image.jpg\" alt=\"Example\">");

		assertThat(plainText).isEqualTo("![Example](https://www.example.com/image.jpg)");
	}

	@DisplayName("should render blockquote")
	@Test
	void shouldRenderBlockquote() {
		var plainText = renderPlainText("<blockquote><p>Example</p>\n<p>Example2</p></blockquote>");

		assertThat(plainText).isEqualTo("> Example Example2");
	}

	@DisplayName("should render html")
	@Test
	void shouldRenderHtml() {
		var plainText = renderPlainText("<p>Example&#128566;</p><br/><p>Example2</p><b>");

		assertThat(plainText).isEqualTo("Example\uD83D\uDE36\nExample2____");
	}
	
	private String renderPlainText(String html) {
		return new Osi2HtmlDocument(html).renderToPlainText();
	}
}